@@ -1,6 +1,7 @@
 use super::*;
 use russell_lab::*;
-use std::fmt::{self, Write};
+use russell_openblas::to_i32;
+use std::fmt;
 
 #[repr(C)]
 pub(crate) struct ExtSolver {
@@ -18,7 +19,7 @@ extern "C" {
         nnz: i32,
         indices_i: *const i32,
         indices_j: *const i32,
-        values_a: *const f64,
+        values_aij: *const f64,
         ordering: i32,
         scaling: i32,
         pct_inc_workspace: i32,
@@ -28,6 +29,8 @@ extern "C" {
     ) -> i32;
     fn solver_mmp_factorize(solver: *mut ExtSolver, verbose: i32) -> i32;
     fn solver_mmp_solve(solver: *mut ExtSolver, rhs: *mut f64, verbose: i32) -> i32;
+    fn solver_mmp_used_ordering(solver: *const ExtSolver) -> i32;
+    fn solver_mmp_used_scaling(solver: *const ExtSolver) -> i32;
 
     // UMF
     fn new_solver_umf(symmetry: i32) -> *mut ExtSolver;
@@ -38,13 +41,15 @@ extern "C" {
         nnz: i32,
         indices_i: *const i32,
         indices_j: *const i32,
-        values_a: *const f64,
+        values_aij: *const f64,
         ordering: i32,
         scaling: i32,
         verbose: i32,
     ) -> i32;
     fn solver_umf_factorize(solver: *mut ExtSolver, verbose: i32) -> i32;
     fn solver_umf_solve(solver: *mut ExtSolver, x: *mut f64, rhs: *const f64, verbose: i32) -> i32;
+    fn solver_umf_used_ordering(solver: *const ExtSolver) -> i32;
+    fn solver_umf_used_scaling(solver: *const ExtSolver) -> i32;
 }
 
 /// Implements a sparse Solver
@@ -57,39 +62,24 @@ extern "C" {
 /// (m,m)   (m)    (m)
 /// ```
 pub struct Solver {
-    config: ConfigSolver,   // configuration
-    done_initialize: bool,  // initialization completed
-    done_factorize: bool,   // factorization completed
-    ndim: usize,            // number of equations == nrow(a) where a*x=rhs
-    solver: *mut ExtSolver, // data allocated by the c-code
-    stopwatch: Stopwatch,   // stopwatch to measure elapsed time
-    time_init: u128,        // elapsed time during initialize
-    time_fact: u128,        // elapsed time during factorize
-    time_solve: u128,       // elapsed time during solve
+    config: ConfigSolver,        // configuration
+    done_initialize: bool,       // initialization completed
+    done_factorize: bool,        // factorization completed
+    ndim: usize,                 // number of equations == nrow(a) where a*x=rhs
+    solver: *mut ExtSolver,      // data allocated by the c-code
+    stopwatch: Stopwatch,        // stopwatch to measure elapsed time
+    time_init: u128,             // elapsed time during initialize
+    time_fact: u128,             // elapsed time during factorize
+    time_solve: u128,            // elapsed time during solve
+    used_ordering: &'static str, // used ordering strategy
+    used_scaling: &'static str,  // used scaling strategy
 }
 
 impl Solver {
     /// Creates a new solver
-    ///
-    /// # Example
-    ///
-    /// ```
-    /// # fn main() -> Result<(), &'static str> {
-    /// use russell_sparse::*;
-    /// let config = ConfigSolver::new();
-    /// let solver = Solver::new(config)?;
-    /// let correct: &str = "solver_kind        = UMF\n\
-    ///                      symmetry           = No\n\
-    ///                      ordering           = Auto\n\
-    ///                      scaling            = Auto\n\
-    ///                      done_initialize    = false\n\
-    ///                      done_factorize     = false\n\
-    ///                      ndim               = 0\n";
-    /// assert_eq!(format!("{}", solver), correct);
-    /// # Ok(())
-    /// # }
-    /// ```
     pub fn new(config: ConfigSolver) -> Result<Self, &'static str> {
+        let used_ordering = str_enum_ordering(config.ordering);
+        let used_scaling = str_enum_scaling(config.scaling);
         unsafe {
             let solver = match config.solver_kind {
                 EnumSolverKind::Mmp => new_solver_mmp(config.symmetry),
@@ -108,34 +98,13 @@ impl Solver {
                 time_init: 0,
                 time_fact: 0,
                 time_solve: 0,
+                used_ordering,
+                used_scaling,
             })
         }
     }
 
     /// Initializes the solver
-    ///
-    /// # Example
-    ///
-    /// ```
-    /// # fn main() -> Result<(), &'static str> {
-    /// use russell_sparse::*;
-    /// let config = ConfigSolver::new();
-    /// let mut solver = Solver::new(config)?;
-    /// let mut trip = SparseTriplet::new(2, 2, 2, false)?;
-    /// trip.put(0, 0, 1.0);
-    /// trip.put(1, 1, 1.0);
-    /// solver.initialize(&trip, false)?;
-    /// let correct: &str = "solver_kind        = UMF\n\
-    ///                      symmetry           = No\n\
-    ///                      ordering           = Auto\n\
-    ///                      scaling            = Auto\n\
-    ///                      done_initialize    = true\n\
-    ///                      done_factorize     = false\n\
-    ///                      ndim               = 2\n";
-    /// assert_eq!(format!("{}", solver), correct);
-    /// # Ok(())
-    /// # }
-    /// ```
     pub fn initialize(&mut self, trip: &SparseTriplet, verbose: bool) -> Result<(), &'static str> {
         if trip.nrow != trip.ncol {
             return Err("the matrix represented by the triplet must be square");
@@ -160,7 +129,7 @@ impl Solver {
                         nnz,
                         trip.indices_i.as_ptr(),
                         trip.indices_j.as_ptr(),
-                        trip.values_a.as_ptr(),
+                        trip.values_aij.as_ptr(),
                         self.config.ordering,
                         self.config.scaling,
                         self.config.pct_inc_workspace,
@@ -186,7 +155,7 @@ impl Solver {
                         nnz,
                         trip.indices_i.as_ptr(),
                         trip.indices_j.as_ptr(),
-                        trip.values_a.as_ptr(),
+                        trip.values_aij.as_ptr(),
                         self.config.ordering,
                         self.config.scaling,
                         verb,
@@ -205,30 +174,6 @@ impl Solver {
     }
 
     /// Performs the factorization
-    ///
-    /// # Example
-    ///
-    /// ```
-    /// # fn main() -> Result<(), &'static str> {
-    /// use russell_sparse::*;
-    /// let config = ConfigSolver::new();
-    /// let mut solver = Solver::new(config)?;
-    /// let mut trip = SparseTriplet::new(2, 2, 2, false)?;
-    /// trip.put(0, 0, 1.0);
-    /// trip.put(1, 1, 1.0);
-    /// solver.initialize(&trip, false)?;
-    /// solver.factorize(false)?;
-    /// let correct: &str = "solver_kind        = UMF\n\
-    ///                      symmetry           = No\n\
-    ///                      ordering           = Auto\n\
-    ///                      scaling            = Auto\n\
-    ///                      done_initialize    = true\n\
-    ///                      done_factorize     = true\n\
-    ///                      ndim               = 2\n";
-    /// assert_eq!(format!("{}", solver), correct);
-    /// # Ok(())
-    /// # }
-    /// ```
     pub fn factorize(&mut self, verbose: bool) -> Result<(), &'static str> {
         if !self.done_initialize {
             return Err("initialization must be done before factorization");
@@ -242,12 +187,20 @@ impl Solver {
                     if res != 0 {
                         return Err(self.handle_mmp_error_code(res));
                     }
+                    let ord = solver_mmp_used_ordering(self.solver);
+                    let sca = solver_mmp_used_scaling(self.solver);
+                    self.used_ordering = str_mmp_ordering(ord);
+                    self.used_scaling = str_mmp_scaling(sca);
                 }
                 EnumSolverKind::Umf => {
                     let res = solver_umf_factorize(self.solver, verb);
                     if res != 0 {
                         return Err(self.handle_umf_error_code(res));
                     }
+                    let ord = solver_umf_used_ordering(self.solver);
+                    let sca = solver_umf_used_scaling(self.solver);
+                    self.used_ordering = str_umf_ordering(ord);
+                    self.used_scaling = str_umf_scaling(sca);
                 }
             }
         }
@@ -266,7 +219,7 @@ impl Solver {
     /// use russell_sparse::*;
     ///
     /// // allocate a square matrix
-    /// let mut trip = SparseTriplet::new(5, 5, 13, false)?;
+    /// let mut trip = SparseTriplet::new(5, 5, 13, false, false)?;
     /// trip.put(0, 0, 1.0); // << (0, 0, a00/2)
     /// trip.put(0, 0, 1.0); // << (0, 0, a00/2)
     /// trip.put(1, 0, 3.0);
@@ -372,7 +325,7 @@ impl Solver {
     /// use russell_sparse::*;
     ///
     /// // allocate a square matrix
-    /// let mut trip = SparseTriplet::new(3, 3, 5, false)?;
+    /// let mut trip = SparseTriplet::new(3, 3, 5, false, false)?;
     /// trip.put(0, 0, 0.2);
     /// trip.put(0, 1, 0.2);
     /// trip.put(1, 0, 0.5);
@@ -431,13 +384,26 @@ impl Solver {
         Ok((solver, x))
     }
 
-    /// Returns a text containing the elapsed times
-    pub fn get_elapsed_times(&self) -> String {
-        let mut buf = String::new();
-        write!(&mut buf, "initialize = {}\n", format_nanoseconds(self.time_init)).unwrap();
-        write!(&mut buf, "factorize  = {}\n", format_nanoseconds(self.time_fact)).unwrap();
-        write!(&mut buf, "solve      = {}\n", format_nanoseconds(self.time_solve)).unwrap();
-        buf
+    /// Returns the elapsed times
+    ///
+    /// # Output
+    ///
+    /// * `(time_init, time_fact, time_solve)` -- elapsed times during initialize, factorize, and solve, respectively
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// # fn main() -> Result<(), &'static str> {
+    /// use russell_sparse::*;
+    /// let config = ConfigSolver::new();
+    /// let solver = Solver::new(config)?;
+    /// let times = solver.get_elapsed_times();
+    /// assert_eq!(times, (0, 0, 0));
+    /// # Ok(())
+    /// # }
+    /// ```
+    pub fn get_elapsed_times(&self) -> (u128, u128, u128) {
+        (self.time_init, self.time_fact, self.time_solve)
     }
 
     /// Handles error code
@@ -560,13 +526,37 @@ impl Drop for Solver {
 
 impl fmt::Display for Solver {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        let time_total = self.time_init + self.time_fact + self.time_solve;
         write!(
             f,
-            "{}\
-             done_initialize    = {}\n\
-             done_factorize     = {}\n\
-             ndim               = {}\n",
-            self.config, self.done_initialize, self.done_factorize, self.ndim,
+            "{},\n\
+             \x20\x20\x20\x20\"usedOrdering\": \"{}\",\n\
+             \x20\x20\x20\x20\"usedScaling\": \"{}\",\n\
+             \x20\x20\x20\x20\"doneInitialize\": {},\n\
+             \x20\x20\x20\x20\"doneFactorize\": {},\n\
+             \x20\x20\x20\x20\"ndim\": {},\n\
+             \x20\x20\x20\x20\"timeInitNs\": {},\n\
+             \x20\x20\x20\x20\"timeFactNs\": {},\n\
+             \x20\x20\x20\x20\"timeSolveNs\": {},\n\
+             \x20\x20\x20\x20\"timeTotalNs\": {},\n\
+             \x20\x20\x20\x20\"timeInitStr\": \"{}\",\n\
+             \x20\x20\x20\x20\"timeFactStr\": \"{}\",\n\
+             \x20\x20\x20\x20\"timeSolveStr\": \"{}\",\n\
+             \x20\x20\x20\x20\"timeTotalStr\": \"{}\"",
+            self.config,
+            self.used_ordering,
+            self.used_scaling,
+            self.done_initialize,
+            self.done_factorize,
+            self.ndim,
+            self.time_init,
+            self.time_fact,
+            self.time_solve,
+            time_total,
+            format_nanoseconds(self.time_init),
+            format_nanoseconds(self.time_fact),
+            format_nanoseconds(self.time_solve),
+            format_nanoseconds(time_total)
         )?;
         Ok(())
     }
@@ -587,26 +577,11 @@ mod tests {
         Ok(())
     }
 
-    #[test]
-    fn display_trait_works() -> Result<(), &'static str> {
-        let config = ConfigSolver::new();
-        let solver = Solver::new(config)?;
-        let correct: &str = "solver_kind        = UMF\n\
-                             symmetry           = No\n\
-                             ordering           = Auto\n\
-                             scaling            = Auto\n\
-                             done_initialize    = false\n\
-                             done_factorize     = false\n\
-                             ndim               = 0\n";
-        assert_eq!(format!("{}", solver), correct);
-        Ok(())
-    }
-
     #[test]
     fn initialize_fails_on_rect_matrix() -> Result<(), &'static str> {
         let config = ConfigSolver::new();
         let mut solver = Solver::new(config)?;
-        let trip_rect = SparseTriplet::new(3, 2, 1, false)?;
+        let trip_rect = SparseTriplet::new(3, 2, 1, false, false)?;
         assert_eq!(
             solver.initialize(&trip_rect, false),
             Err("the matrix represented by the triplet must be square")
@@ -618,7 +593,7 @@ mod tests {
     fn initialize_works() -> Result<(), &'static str> {
         let config = ConfigSolver::new();
         let mut solver = Solver::new(config)?;
-        let mut trip = SparseTriplet::new(2, 2, 2, false)?;
+        let mut trip = SparseTriplet::new(2, 2, 2, false, false)?;
         trip.put(0, 0, 1.0);
         trip.put(1, 1, 1.0);
         solver.initialize(&trip, false)?;
@@ -641,7 +616,7 @@ mod tests {
     fn factorize_fails_on_singular_matrix() -> Result<(), &'static str> {
         let config = ConfigSolver::new();
         let mut solver = Solver::new(config)?;
-        let mut trip = SparseTriplet::new(2, 2, 2, false)?;
+        let mut trip = SparseTriplet::new(2, 2, 2, false, false)?;
         trip.put(0, 0, 1.0);
         trip.put(1, 1, 0.0);
         solver.initialize(&trip, false)?;
@@ -653,7 +628,7 @@ mod tests {
     fn factorize_works() -> Result<(), &'static str> {
         let config = ConfigSolver::new();
         let mut solver = Solver::new(config)?;
-        let mut trip = SparseTriplet::new(2, 2, 2, false)?;
+        let mut trip = SparseTriplet::new(2, 2, 2, false, false)?;
         trip.put(0, 0, 1.0);
         trip.put(1, 1, 1.0);
         solver.initialize(&trip, false)?;
@@ -666,7 +641,7 @@ mod tests {
     fn solve_fails_on_non_factorized() -> Result<(), &'static str> {
         let config = ConfigSolver::new();
         let mut solver = Solver::new(config)?;
-        let mut trip = SparseTriplet::new(2, 2, 2, false)?;
+        let mut trip = SparseTriplet::new(2, 2, 2, false, false)?;
         trip.put(0, 0, 1.0);
         trip.put(1, 1, 1.0);
         solver.initialize(&trip, false)?;
@@ -683,7 +658,7 @@ mod tests {
     fn solve_fails_on_wrong_vectors() -> Result<(), &'static str> {
         let config = ConfigSolver::new();
         let mut solver = Solver::new(config)?;
-        let mut trip = SparseTriplet::new(2, 2, 2, false)?;
+        let mut trip = SparseTriplet::new(2, 2, 2, false, false)?;
         trip.put(0, 0, 1.0);
         trip.put(1, 1, 1.0);
         solver.initialize(&trip, false)?;
@@ -709,7 +684,7 @@ mod tests {
         let mut solver = Solver::new(config)?;
 
         // allocate a square matrix
-        let mut trip = SparseTriplet::new(5, 5, 13, false)?;
+        let mut trip = SparseTriplet::new(5, 5, 13, false, false)?;
         trip.put(0, 0, 1.0); // << (0, 0, a00/2)
         trip.put(0, 0, 1.0); // << (0, 0, a00/2)
         trip.put(1, 0, 3.0);
@@ -743,7 +718,7 @@ mod tests {
     fn reinitialize_works() -> Result<(), &'static str> {
         let config = ConfigSolver::new();
         let mut solver = Solver::new(config)?;
-        let mut trip = SparseTriplet::new(2, 2, 2, false)?;
+        let mut trip = SparseTriplet::new(2, 2, 2, false, false)?;
         trip.put(0, 0, 1.0);
         trip.put(1, 1, 1.0);
         solver.initialize(&trip, false)?;
@@ -767,7 +742,7 @@ mod tests {
         let mut solver = Solver::new(config)?;
 
         // initialize fails on rectangular matrix
-        let trip_rect = SparseTriplet::new(3, 2, 1, false)?;
+        let trip_rect = SparseTriplet::new(3, 2, 1, false, false)?;
         assert_eq!(
             solver.initialize(&trip_rect, false),
             Err("the matrix represented by the triplet must be square")
@@ -780,7 +755,7 @@ mod tests {
         );
 
         // allocate a square matrix
-        let mut trip = SparseTriplet::new(5, 5, 13, false)?;
+        let mut trip = SparseTriplet::new(5, 5, 13, false, false)?;
         trip.put(0, 0, 1.0); // << (0, 0, a00/2)
         trip.put(0, 0, 1.0); // << (0, 0, a00/2)
         trip.put(1, 0, 3.0);
@@ -838,7 +813,7 @@ mod tests {
         assert_vec_approx_eq!(x_again.as_data(), x_correct, 1e-14);
 
         // factorize fails on singular matrix
-        let mut trip_singular = SparseTriplet::new(5, 5, 2, false)?;
+        let mut trip_singular = SparseTriplet::new(5, 5, 2, false, false)?;
         trip_singular.put(0, 0, 1.0);
         trip_singular.put(4, 4, 1.0);
         solver.initialize(&trip_singular, false)?;
@@ -850,7 +825,7 @@ mod tests {
 
     #[test]
     fn new_solution_works() -> Result<(), &'static str> {
-        let mut trip = SparseTriplet::new(3, 3, 6, false)?;
+        let mut trip = SparseTriplet::new(3, 3, 6, false, false)?;
         trip.put(0, 0, 1.0);
         trip.put(0, 1, 1.0);
         trip.put(1, 0, 2.0);
@@ -872,10 +847,7 @@ mod tests {
         let config = ConfigSolver::new();
         let solver = Solver::new(config)?;
         let times = solver.get_elapsed_times();
-        let correct: &str = "initialize = 0ns\n\
-                             factorize  = 0ns\n\
-                             solve      = 0ns\n";
-        assert_eq!(format!("{}", times), correct);
+        assert_eq!(times, (0, 0, 0));
         Ok(())
     }
 
@@ -933,4 +905,32 @@ mod tests {
         assert_eq!(solver.handle_umf_error_code(123), default);
         Ok(())
     }
+
+    #[test]
+    fn display_trait_works() -> Result<(), &'static str> {
+        let config = ConfigSolver::new();
+        let solver = Solver::new(config)?;
+        let correct: &str = "\x20\x20\x20\x20\"solverKind\": \"UMF\",\n\
+                             \x20\x20\x20\x20\"symmetry\": \"No\",\n\
+                             \x20\x20\x20\x20\"ordering\": \"Auto\",\n\
+                             \x20\x20\x20\x20\"scaling\": \"Auto\",\n\
+                             \x20\x20\x20\x20\"pctIncWorkspace\": 100,\n\
+                             \x20\x20\x20\x20\"maxWorkMemory\": 0,\n\
+                             \x20\x20\x20\x20\"openmpNumThreads\": 1,\n\
+                             \x20\x20\x20\x20\"usedOrdering\": \"Auto\",\n\
+                             \x20\x20\x20\x20\"usedScaling\": \"Auto\",\n\
+                             \x20\x20\x20\x20\"doneInitialize\": false,\n\
+                             \x20\x20\x20\x20\"doneFactorize\": false,\n\
+                             \x20\x20\x20\x20\"ndim\": 0,\n\
+                             \x20\x20\x20\x20\"timeInitNs\": 0,\n\
+                             \x20\x20\x20\x20\"timeFactNs\": 0,\n\
+                             \x20\x20\x20\x20\"timeSolveNs\": 0,\n\
+                             \x20\x20\x20\x20\"timeTotalNs\": 0,\n\
+                             \x20\x20\x20\x20\"timeInitStr\": \"0ns\",\n\
+                             \x20\x20\x20\x20\"timeFactStr\": \"0ns\",\n\
+                             \x20\x20\x20\x20\"timeSolveStr\": \"0ns\",\n\
+                             \x20\x20\x20\x20\"timeTotalStr\": \"0ns\"";
+        assert_eq!(format!("{}", solver), correct);
+        Ok(())
+    }
 }
