@@ -1,5 +1,5 @@
-use super::*;
-use russell_lab::*;
+use russell_lab::{Matrix, Vector};
+use russell_openblas::to_i32;
 use std::fmt;
 
 /// Holds triples (i,j,aij) representing a sparse matrix
@@ -14,14 +14,15 @@ use std::fmt;
 /// - The maximum number of entries includes possible entries with repeated indices
 /// - See the `to_matrix` method for an example
 pub struct SparseTriplet {
-    pub(crate) nrow: usize,         // [i32] number of rows
-    pub(crate) ncol: usize,         // [i32] number of columns
-    pub(crate) pos: usize,          // [i32] current index => nnz in the end
-    pub(crate) max: usize,          // [i32] max allowed number of entries (may be > nnz)
-    pub(crate) symmetric: bool,     // general symmetric?, but WITHOUT one side of the diagonal
-    pub(crate) indices_i: Vec<i32>, // [nnz] indices i
-    pub(crate) indices_j: Vec<i32>, // [nnz] indices j
-    pub(crate) values_a: Vec<f64>,  // [nnz] values a
+    pub(crate) nrow: usize,          // [i32] number of rows
+    pub(crate) ncol: usize,          // [i32] number of columns
+    pub(crate) pos: usize,           // [i32] current index => nnz in the end
+    pub(crate) max: usize,           // [i32] max allowed number of entries (may be > nnz)
+    pub(crate) sym_part: bool,       // general symmetric, without one side of the diagonal (lower or upper)
+    pub(crate) sym_full: bool,       // general symmetric, with both sides of the diagonal
+    pub(crate) indices_i: Vec<i32>,  // [nnz] indices i
+    pub(crate) indices_j: Vec<i32>,  // [nnz] indices j
+    pub(crate) values_aij: Vec<f64>, // [nnz] values aij
 }
 
 impl SparseTriplet {
@@ -38,58 +39,29 @@ impl SparseTriplet {
     /// * `ncol` -- The number of columns of the sparse matrix
     /// * `max` -- The maximum number fo non-zero values in the sparse matrix,
     ///            including entries with repeated indices
-    /// * `symmetric` -- This Triplet represents a **general** symmetric matrix.
-    ///                  In this case, one side of the diagonal may be ignored.
-    ///
-    /// # Example
-    ///
-    /// ```
-    /// # fn main() -> Result<(), &'static str> {
-    /// use russell_sparse::*;
-    /// let trip = SparseTriplet::new(3, 3, 5, false)?;
-    /// let correct: &str = "nrow      = 3\n\
-    ///                      ncol      = 3\n\
-    ///                      max       = 5\n\
-    ///                      pos       = 0\n\
-    ///                      symmetric = false\n";
-    /// assert_eq!(format!("{}", trip), correct);
-    /// # Ok(())
-    /// # }
-    /// ```
-    pub fn new(nrow: usize, ncol: usize, max: usize, symmetric: bool) -> Result<Self, &'static str> {
+    /// * `sym_part` -- The matrix is symmetric, but represented by the lower- or upper-triangular part only (e.g., for the MMP solver)
+    /// * `sym_full` -- The matrix is symmetric and has both sides of the diagonal represented (e.g., for the UMF solver)
+    pub fn new(nrow: usize, ncol: usize, max: usize, sym_part: bool, sym_full: bool) -> Result<Self, &'static str> {
         if nrow == 0 || ncol == 0 || max == 0 {
             return Err("nrow, ncol, and max must all be greater than zero");
         }
+        if sym_part && sym_full {
+            return Err("either sym_part xor sym_full may be true, but not both");
+        }
         Ok(SparseTriplet {
             nrow,
             ncol,
             pos: 0,
             max,
-            symmetric,
+            sym_part,
+            sym_full,
             indices_i: vec![0; max],
             indices_j: vec![0; max],
-            values_a: vec![0.0; max],
+            values_aij: vec![0.0; max],
         })
     }
 
     /// Puts the next triple (i,j,aij) into the Triplet
-    ///
-    /// # Example
-    ///
-    /// ```
-    /// # fn main() -> Result<(), &'static str> {
-    /// use russell_sparse::*;
-    /// let mut trip = SparseTriplet::new(2, 2, 1, false)?;
-    /// trip.put(0, 0, 1.0);
-    /// let correct: &str = "nrow      = 2\n\
-    ///                      ncol      = 2\n\
-    ///                      max       = 1\n\
-    ///                      pos       = 1 (FULL)\n\
-    ///                      symmetric = false\n";
-    /// assert_eq!(format!("{}", trip), correct);
-    /// # Ok(())
-    /// # }
-    /// ```
     pub fn put(&mut self, i: usize, j: usize, aij: f64) {
         assert!(i < self.nrow);
         assert!(j < self.ncol);
@@ -98,7 +70,7 @@ impl SparseTriplet {
         let j_i32 = to_i32(j);
         self.indices_i[self.pos] = i_i32;
         self.indices_j[self.pos] = j_i32;
-        self.values_a[self.pos] = aij;
+        self.values_aij[self.pos] = aij;
         self.pos += 1;
     }
 
@@ -114,7 +86,7 @@ impl SparseTriplet {
     /// ```
     /// # fn main() -> Result<(), &'static str> {
     /// use russell_sparse::*;
-    /// let trip = SparseTriplet::new(2, 2, 1, false)?;
+    /// let trip = SparseTriplet::new(2, 2, 1, false, false)?;
     /// assert_eq!(trip.dims(), (2, 2));
     /// # Ok(())
     /// # }
@@ -123,20 +95,26 @@ impl SparseTriplet {
         (self.nrow, self.ncol)
     }
 
-    /// Returns the value of the symmetric flag
+    /// Returns the value of the symmetric flags
+    ///
+    /// # Output
+    ///
+    /// `(sym_part, sym_full)` -- where `sym_part` means a symmetric matrix with the
+    ///                           lower- (upper-) part only and `sym_full` means a
+    ///                           symmetric matrix with both sides of the diagonal
     ///
     /// # Example
     ///
     /// ```
     /// # fn main() -> Result<(), &'static str> {
     /// use russell_sparse::*;
-    /// let trip = SparseTriplet::new(2, 2, 1, true)?;
-    /// assert_eq!(trip.is_symmetric(), true);
+    /// let trip = SparseTriplet::new(2, 2, 1, true, false)?;
+    /// assert_eq!(trip.is_symmetric(), (true, false));
     /// # Ok(())
     /// # }
     /// ```
-    pub fn is_symmetric(&self) -> bool {
-        self.symmetric
+    pub fn is_symmetric(&self) -> (bool, bool) {
+        (self.sym_part, self.sym_full)
     }
 
     /// Converts the triples data to a matrix, up to a limit
@@ -155,7 +133,7 @@ impl SparseTriplet {
     ///
     /// // define (4 x 4) sparse matrix with 6+1 non-zero values
     /// // (with an extra ij-repeated entry)
-    /// let mut trip = SparseTriplet::new(4, 4, 6+1, false)?;
+    /// let mut trip = SparseTriplet::new(4, 4, 6+1, false, false)?;
     /// trip.put(0, 0, 0.5); // (0, 0, a00/2)
     /// trip.put(0, 0, 0.5); // (0, 0, a00/2)
     /// trip.put(0, 1, 2.0);
@@ -197,28 +175,97 @@ impl SparseTriplet {
         a.fill(0.0);
         for p in 0..self.pos {
             if self.indices_i[p] < m_i32 && self.indices_j[p] < n_i32 {
-                a.plus_equal(self.indices_i[p] as usize, self.indices_j[p] as usize, self.values_a[p]);
+                let (i, j) = (self.indices_i[p] as usize, self.indices_j[p] as usize);
+                a[i][j] += self.values_aij[p];
             }
         }
         Ok(())
     }
+
+    /// Performs the matrix-vector multiplication
+    ///
+    /// ```text
+    ///  v  :=   a   ⋅  u
+    /// (m)    (m,n)   (n)
+    /// ```
+    ///
+    /// # Note
+    ///
+    /// This method is not highly efficient but should useful in verifications.
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// # fn main() -> Result<(), &'static str> {
+    /// // import
+    /// use russell_lab::*;
+    /// use russell_sparse::*;
+    ///
+    /// // set sparse matrix (4 x 3) with 6 non-zeros
+    /// let mut trip = SparseTriplet::new(4, 3, 6, false, false)?;
+    /// trip.put(0, 0, 1.0);
+    /// trip.put(1, 0, 2.0);
+    /// trip.put(1, 1, 3.0);
+    /// trip.put(2, 0, 4.0);
+    /// trip.put(3, 0, 5.0);
+    /// trip.put(3, 2, 6.0);
+    ///
+    /// // check matrix
+    /// let (m, n) = trip.dims();
+    /// let mut a = Matrix::new(m, n);
+    /// trip.to_matrix(&mut a)?;
+    /// let correct_a = "┌       ┐\n\
+    ///                  │ 1 0 0 │\n\
+    ///                  │ 2 3 0 │\n\
+    ///                  │ 4 0 0 │\n\
+    ///                  │ 5 0 6 │\n\
+    ///                  └       ┘";
+    /// assert_eq!(format!("{}", a), correct_a);
+    ///
+    /// // perform mat-vec-mul
+    /// let u = Vector::from(&[1.0, 1.0, 1.0]);
+    /// let v = trip.mat_vec_mul(&u)?;
+    ///
+    /// // check vector
+    /// let correct_v = "┌    ┐\n\
+    ///                  │  1 │\n\
+    ///                  │  5 │\n\
+    ///                  │  4 │\n\
+    ///                  │ 11 │\n\
+    ///                  └    ┘";
+    /// assert_eq!(format!("{}", v), correct_v);
+    /// # Ok(())
+    /// # }
+    /// ```
+    pub fn mat_vec_mul(&self, u: &Vector) -> Result<Vector, &'static str> {
+        if u.dim() != self.ncol {
+            return Err("u.ndim must equal a.ncol");
+        }
+        let mut v = Vector::new(self.nrow);
+        for p in 0..self.pos {
+            let i = self.indices_i[p] as usize;
+            let j = self.indices_j[p] as usize;
+            let aij = self.values_aij[p];
+            v[i] += aij * u[j];
+            if self.sym_part && i != j {
+                v[j] += aij * u[i];
+            }
+        }
+        Ok(v)
+    }
 }
 
 impl fmt::Display for SparseTriplet {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        let pos = if self.pos == self.max {
-            format!("{} (FULL)", self.pos)
-        } else {
-            format!("{}", self.pos)
-        };
         write!(
             f,
-            "nrow      = {}\n\
-             ncol      = {}\n\
-             max       = {}\n\
-             pos       = {}\n\
-             symmetric = {}\n",
-            self.nrow, self.ncol, self.max, pos, self.symmetric
+            "\x20\x20\x20\x20\"nrow\": {},\n\
+             \x20\x20\x20\x20\"ncol\": {},\n\
+             \x20\x20\x20\x20\"pos\": {},\n\
+             \x20\x20\x20\x20\"max\": {},\n\
+             \x20\x20\x20\x20\"symPart\": {},\n\
+             \x20\x20\x20\x20\"symFull\": {}",
+            self.nrow, self.ncol, self.pos, self.max, self.sym_part, self.sym_full
         )?;
         Ok(())
     }
@@ -228,79 +275,67 @@ impl fmt::Display for SparseTriplet {
 
 #[cfg(test)]
 mod tests {
+    use russell_chk::assert_vec_approx_eq;
+
     use super::*;
 
     #[test]
-    fn new_fails_on_wrong_dims() {
+    fn new_fails_on_wrong_input() {
         assert_eq!(
-            SparseTriplet::new(0, 3, 5, false).err(),
+            SparseTriplet::new(0, 3, 5, false, false).err(),
             Some("nrow, ncol, and max must all be greater than zero")
         );
         assert_eq!(
-            SparseTriplet::new(3, 0, 5, false).err(),
+            SparseTriplet::new(3, 0, 5, false, false).err(),
             Some("nrow, ncol, and max must all be greater than zero")
         );
         assert_eq!(
-            SparseTriplet::new(3, 3, 0, false).err(),
+            SparseTriplet::new(3, 3, 0, false, false).err(),
             Some("nrow, ncol, and max must all be greater than zero")
         );
+        assert_eq!(
+            SparseTriplet::new(1, 1, 1, true, true).err(),
+            Some("either sym_part xor sym_full may be true, but not both")
+        )
     }
 
     #[test]
     fn new_works() -> Result<(), &'static str> {
-        let trip = SparseTriplet::new(3, 3, 5, false)?;
+        let trip = SparseTriplet::new(3, 3, 5, false, false)?;
         assert_eq!(trip.nrow, 3);
         assert_eq!(trip.ncol, 3);
         assert_eq!(trip.pos, 0);
         assert_eq!(trip.max, 5);
-        assert_eq!(trip.symmetric, false);
-        Ok(())
-    }
-
-    #[test]
-    fn display_trait_works() -> Result<(), &'static str> {
-        let mut trip = SparseTriplet::new(3, 3, 1, false)?;
-        let correct_1: &str = "nrow      = 3\n\
-                               ncol      = 3\n\
-                               max       = 1\n\
-                               pos       = 0\n\
-                               symmetric = false\n";
-        assert_eq!(format!("{}", trip), correct_1);
-        trip.put(0, 0, 1.0);
-        let correct_2: &str = "nrow      = 3\n\
-                               ncol      = 3\n\
-                               max       = 1\n\
-                               pos       = 1 (FULL)\n\
-                               symmetric = false\n";
-        assert_eq!(format!("{}", trip), correct_2);
+        assert_eq!(trip.sym_part, false);
+        assert_eq!(trip.sym_full, false);
         Ok(())
     }
 
     #[test]
     #[should_panic]
     fn put_panics_on_wrong_values_1() {
-        let mut trip = SparseTriplet::new(1, 1, 1, false).unwrap();
+        let mut trip = SparseTriplet::new(1, 1, 1, false, false).unwrap();
         trip.put(1, 0, 0.0);
     }
 
     #[test]
     #[should_panic]
     fn put_panics_on_wrong_values_2() {
-        let mut trip = SparseTriplet::new(1, 1, 1, false).unwrap();
+        let mut trip = SparseTriplet::new(1, 1, 1, false, false).unwrap();
         trip.put(0, 1, 0.0);
     }
 
     #[test]
     #[should_panic]
     fn put_panics_on_wrong_values_3() {
-        let mut trip = SparseTriplet::new(1, 1, 1, false).unwrap();
+        let mut trip = SparseTriplet::new(1, 1, 1, false, false).unwrap();
         trip.put(0, 0, 0.0); // << all spots occupied
         trip.put(0, 0, 0.0);
     }
 
     #[test]
     fn put_works() -> Result<(), &'static str> {
-        let mut trip = SparseTriplet::new(3, 3, 5, false)?;
+        let mut trip = SparseTriplet::new(3, 3, 5, false, false)?;
         trip.put(0, 0, 1.0);
         assert_eq!(trip.pos, 1);
         trip.put(0, 1, 2.0);
@@ -316,21 +351,23 @@ mod tests {
 
     #[test]
     fn dims_works() -> Result<(), &'static str> {
-        let trip = SparseTriplet::new(3, 2, 1, false)?;
+        let trip = SparseTriplet::new(3, 2, 1, false, false)?;
         assert_eq!(trip.dims(), (3, 2));
         Ok(())
     }
 
     #[test]
     fn is_symmetric_works() -> Result<(), &'static str> {
-        let trip = SparseTriplet::new(3, 2, 1, true)?;
-        assert_eq!(trip.is_symmetric(), true);
+        let trip1 = SparseTriplet::new(1, 1, 1, true, false)?;
+        let trip2 = SparseTriplet::new(1, 1, 1, false, true)?;
+        assert_eq!(trip1.is_symmetric(), (true, false));
+        assert_eq!(trip2.is_symmetric(), (false, true));
         Ok(())
     }
 
     #[test]
     fn to_matrix_fails_on_wrong_dims() -> Result<(), &'static str> {
-        let trip = SparseTriplet::new(1, 1, 1, false)?;
+        let trip = SparseTriplet::new(1, 1, 1, false, false)?;
         let mut a_2x1 = Matrix::new(2, 1);
         let mut a_1x2 = Matrix::new(1, 2);
         assert_eq!(trip.to_matrix(&mut a_2x1), Err("wrong matrix dimensions"));
@@ -340,7 +377,7 @@ mod tests {
 
     #[test]
     fn to_matrix_works() -> Result<(), &'static str> {
-        let mut trip = SparseTriplet::new(3, 3, 5, false)?;
+        let mut trip = SparseTriplet::new(3, 3, 5, false, false)?;
         trip.put(0, 0, 1.0);
         trip.put(0, 1, 2.0);
         trip.put(1, 0, 3.0);
@@ -363,7 +400,7 @@ mod tests {
     #[test]
     fn to_matrix_with_duplicates_works() -> Result<(), &'static str> {
         // allocate a square matrix
-        let mut trip = SparseTriplet::new(5, 5, 13, false)?;
+        let mut trip = SparseTriplet::new(5, 5, 13, false, false)?;
         trip.put(0, 0, 1.0); // << (0, 0, a00/2)
         trip.put(0, 0, 1.0); // << (0, 0, a00/2)
         trip.put(1, 0, 3.0);
@@ -392,4 +429,131 @@ mod tests {
         assert_eq!(format!("{}", a), correct);
         Ok(())
     }
+
+    #[test]
+    fn mat_vec_mul_fails_on_wrong_input() -> Result<(), &'static str> {
+        let trip = SparseTriplet::new(2, 2, 1, false, false)?;
+        let u = Vector::new(3);
+        assert_eq!(trip.mat_vec_mul(&u).err(), Some("u.ndim must equal a.ncol"));
+        Ok(())
+    }
+
+    #[test]
+    fn mat_vec_mul_works() -> Result<(), &'static str> {
+        //  1.0  2.0  3.0  4.0  5.0
+        //  0.1  0.2  0.3  0.4  0.5
+        // 10.0 20.0 30.0 40.0 50.0
+        let mut trip = SparseTriplet::new(3, 5, 15, false, false)?;
+        trip.put(0, 0, 1.0);
+        trip.put(0, 1, 2.0);
+        trip.put(0, 2, 3.0);
+        trip.put(0, 3, 4.0);
+        trip.put(0, 4, 5.0);
+        trip.put(1, 0, 0.1);
+        trip.put(1, 1, 0.2);
+        trip.put(1, 2, 0.3);
+        trip.put(1, 3, 0.4);
+        trip.put(1, 4, 0.5);
+        trip.put(2, 0, 10.0);
+        trip.put(2, 1, 20.0);
+        trip.put(2, 2, 30.0);
+        trip.put(2, 3, 40.0);
+        trip.put(2, 4, 50.0);
+        let u = Vector::from(&[0.1, 0.2, 0.3, 0.4, 0.5]);
+        let correct_v = &[5.5, 0.55, 55.0];
+        let v = trip.mat_vec_mul(&u)?;
+        assert_vec_approx_eq!(v.as_data(), correct_v, 1e-15);
+        Ok(())
+    }
+
+    #[test]
+    fn mat_vec_mul_sym_part_works() -> Result<(), &'static str> {
+        // 2
+        // 1  2     sym
+        // 1  2  9
+        // 3  1  1  7
+        // 2  1  5  1  8
+        let mut trip = SparseTriplet::new(5, 5, 15, true, false)?;
+        trip.put(0, 0, 2.0);
+        trip.put(1, 1, 2.0);
+        trip.put(2, 2, 9.0);
+        trip.put(3, 3, 7.0);
+        trip.put(4, 4, 8.0);
+
+        trip.put(1, 0, 1.0);
+
+        trip.put(2, 0, 1.0);
+        trip.put(2, 1, 2.0);
+
+        trip.put(3, 0, 3.0);
+        trip.put(3, 1, 1.0);
+        trip.put(3, 2, 1.0);
+
+        trip.put(4, 0, 2.0);
+        trip.put(4, 1, 1.0);
+        trip.put(4, 2, 5.0);
+        trip.put(4, 3, 1.0);
+        let u = Vector::from(&[-629.0 / 98.0, 237.0 / 49.0, -53.0 / 49.0, 62.0 / 49.0, 23.0 / 14.0]);
+        let correct_v = &[-2.0, 4.0, 3.0, -5.0, 1.0];
+        let v = trip.mat_vec_mul(&u)?;
+        assert_vec_approx_eq!(v.as_data(), correct_v, 1e-14);
+        Ok(())
+    }
+
+    #[test]
+    fn mat_vec_mul_sym_full_works() -> Result<(), &'static str> {
+        // 2  1  1  3  2
+        // 1  2  2  1  1
+        // 1  2  9  1  5
+        // 3  1  1  7  1
+        // 2  1  5  1  8
+        let mut trip = SparseTriplet::new(5, 5, 25, false, true)?;
+        trip.put(0, 0, 2.0);
+        trip.put(1, 1, 2.0);
+        trip.put(2, 2, 9.0);
+        trip.put(3, 3, 7.0);
+        trip.put(4, 4, 8.0);
+
+        trip.put(1, 0, 1.0);
+        trip.put(0, 1, 1.0);
+
+        trip.put(2, 0, 1.0);
+        trip.put(0, 2, 1.0);
+        trip.put(2, 1, 2.0);
+        trip.put(1, 2, 2.0);
+
+        trip.put(3, 0, 3.0);
+        trip.put(0, 3, 3.0);
+        trip.put(3, 1, 1.0);
+        trip.put(1, 3, 1.0);
+        trip.put(3, 2, 1.0);
+        trip.put(2, 3, 1.0);
+
+        trip.put(4, 0, 2.0);
+        trip.put(0, 4, 2.0);
+        trip.put(4, 1, 1.0);
+        trip.put(1, 4, 1.0);
+        trip.put(4, 2, 5.0);
+        trip.put(2, 4, 5.0);
+        trip.put(4, 3, 1.0);
+        trip.put(3, 4, 1.0);
+        let u = Vector::from(&[-629.0 / 98.0, 237.0 / 49.0, -53.0 / 49.0, 62.0 / 49.0, 23.0 / 14.0]);
+        let correct_v = &[-2.0, 4.0, 3.0, -5.0, 1.0];
+        let v = trip.mat_vec_mul(&u)?;
+        assert_vec_approx_eq!(v.as_data(), correct_v, 1e-14);
+        Ok(())
+    }
+
+    #[test]
+    fn display_trait_works() -> Result<(), &'static str> {
+        let trip = SparseTriplet::new(3, 3, 1, false, true)?;
+        let correct: &str = "\x20\x20\x20\x20\"nrow\": 3,\n\
+                             \x20\x20\x20\x20\"ncol\": 3,\n\
+                             \x20\x20\x20\x20\"pos\": 0,\n\
+                             \x20\x20\x20\x20\"max\": 1,\n\
+                             \x20\x20\x20\x20\"symPart\": false,\n\
+                             \x20\x20\x20\x20\"symFull\": true";
+        assert_eq!(format!("{}", trip), correct);
+        Ok(())
+    }
 }
